
#include "bildspeicher.hh"

#include <qbitmap.h>
#include <fstream.h>

#include "land.hh"
#include "main_win.hh"
#include "depot.hh"

#define type_la	0
#define type_de	1
#define type_fa	2
#define type_hq	3
#define type_du	4

bildspeicher::bildspeicher(main_win *m) {
  
  int		i;
  QBitmap	mask;
  char		fname[512];
  
  mw=m;
  
  land_pxm=NULL;
  land_type=NULL;
  land_spieler=NULL;
  land_anzahl=0;
  fahrzeuge_spieler1=NULL;
  fahrzeuge_spieler2=NULL;
  fahrzeug_anzahl=0;
  einheit_beschreibung=NULL;
  preis=NULL;
  rahmen=NULL;
  moved=NULL;
  stellplatz=NULL;
  strasse=NULL;
  strasse_anzahl=0;
  
  rahmen=new QPixmap("pics/hintergrund.ppm");
  mask.load("pics/rahmen_mask.pbm");
  rahmen->setMask(mask);
  
  moved=new QPixmap("pics/hintergrund.ppm");
  mask.load("pics/zug_beendet_mask.pbm");
  moved->setMask(mask);
  
  stellplatz=new QPixmap("pics/stellplatz.ppm");
  mask.load("pics/hexagon_mask.pbm");
  stellplatz->setMask(mask);
  
  readLandBeschreibung();
  readStrassenBeschreibung();
  readEinheitenBeschreibung();
  
  // Preise laden
  
  strcpy(fname,levelName);
  strcat(fname,"/cost");
  preis=new int [fahrzeug_anzahl];
  ifstream	pr_dat(fname);
  if (!pr_dat) { cerr<<"cant open "<<fname<<endl; exit(1); }
  for (i=0; i<fahrzeug_anzahl; i++) pr_dat>>preis[i];
  
  mask.load("pics/hexagon_mask.pbm");
  for (i=0; i<land_anzahl; i++)
    land_pxm[i]->setMask(mask);
  
  for (i=0; i<fahrzeug_anzahl; i++) {
    fahrzeuge_spieler1[i]->setMask(fahrzeuge_spieler1[i]->createHeuristicMask());
    fahrzeuge_spieler2[i]->setMask(fahrzeuge_spieler2[i]->createHeuristicMask());
  }
  
  for (i=0; i<strasse_anzahl; i++)
    strasse[i]->setMask(strasse[i]->createHeuristicMask());
  
}

bildspeicher::~bildspeicher() {
  
  int		i;
  
  if (land_pxm!=NULL) {
    for (i=0; i<land_anzahl; i++)
      if (land_pxm[i]!=NULL) delete land_pxm[i];
    delete land_pxm;
  }
  if (land_type!=NULL) delete land_type;
  if (fahrzeuge_spieler1!=NULL) {
    for (i=0; i<fahrzeug_anzahl; i++)
      if (fahrzeuge_spieler1[i]!=NULL) delete fahrzeuge_spieler1[i];
    delete fahrzeuge_spieler1;
  }
  if (fahrzeuge_spieler2!=NULL) {
    for (i=0; i<fahrzeug_anzahl; i++)
      if (fahrzeuge_spieler2[i]!=NULL) delete fahrzeuge_spieler2[i];
    delete fahrzeuge_spieler2;
  }
  if (strasse!=NULL) {
    for (i=0; i<strasse_anzahl; i++)
      if (strasse[i]!=NULL) delete strasse[i];
    delete strasse;
  }
  if (preis!=NULL) delete preis;
  if (befahrbarkeit!=NULL) delete befahrbarkeit;
  if (einheit_beschreibung!=NULL) {
    for (i=0; i<fahrzeug_anzahl; i++)
      if (einheit_beschreibung[i]!=NULL) delete einheit_beschreibung[i];
    delete einheit_beschreibung;
  }
  
}

QPixmap *bildspeicher::get_rahmen() {
  return rahmen;
}

QPixmap *bildspeicher::get_moved() {
  return moved;
}

QPixmap *bildspeicher::getFahrzeug(int typ, int spieler) {
  if ((typ>=0) && (typ<fahrzeug_anzahl)) {
    if (spieler==1) return fahrzeuge_spieler1[typ];
    else if (spieler==2) return fahrzeuge_spieler2[typ];
    else { cerr<<"bildspeicher::getFahrzeug() unbekannter Spieler\n"; exit(1); }
  } else return NULL;
}

int bildspeicher::getFahrzeugPreis(int typ) {
  
  if ((typ>=0) && (typ<fahrzeug_anzahl)) return preis[typ];
  else return 0;
  
}

QPixmap *bildspeicher::getStrasse(int str) {
  if ((str>=0) && (str<strasse_anzahl)) return strasse[str];
  else { cerr<<"type strasse out of range\n"; exit(1); }
}

void bildspeicher::readLandBeschreibung() {
  
  char		fname[512];
  
  strcpy(fname,levelName);
  strcat(fname,"/land_description");
  ifstream	in(fname);
  if (!in) { cerr<<"cant open "<<fname<<endl; exit(1); }
  
  int		i;
  char		name[256];
  
  in>>land_anzahl;
  
  land_pxm=new QPixmap * [land_anzahl];
  for (i=0; i<land_anzahl; i++) land_pxm[i]=NULL;
  
  befahrbarkeit=new double [land_anzahl];
  land_type=new int [land_anzahl];
  land_spieler=new int [land_anzahl];
  
  for (i=0; i<land_anzahl; i++) {
    in>>name;
    land_pxm[i]=new QPixmap();
    if (land_pxm[i]->load(name)==FALSE) { cerr<<"cant load "<<name<<endl; exit(1); }
    in>>befahrbarkeit[i];
    in>>name;
    if (strcasecmp(name,"la")==0) land_type[i]=type_la;
    else if (strcasecmp(name,"de")==0) land_type[i]=type_de;
    else if (strcasecmp(name,"fa")==0) land_type[i]=type_fa;
    else if (strcasecmp(name,"hq")==0) land_type[i]=type_hq;
    else if (strcasecmp(name,"du")==0) land_type[i]=type_du;
    else { cerr<<"type in "<<fname<<"  must be la, de, fa or hq\n"; exit(1); }
    in>>land_spieler[i];
  }
  
}

land *bildspeicher::newLand(int type, int px, int py) {
  
  land		*l;
  
  if ((type>=0) && (type<land_anzahl)) {
    switch (land_type[type]) {
      case type_la:
        l=new land(mw,land_pxm[type],befahrbarkeit[type],FALSE);
        break;
      case type_de:
        l=new depot(mw,land_pxm[type],befahrbarkeit[type],land_spieler[type],px,py,FALSE,FALSE);
        break;
      case type_fa:
        l=new depot(mw,land_pxm[type],befahrbarkeit[type],land_spieler[type],px,py,TRUE,FALSE);
        break;
      case type_hq:
        l=new depot(mw,land_pxm[type],befahrbarkeit[type],land_spieler[type],px,py,FALSE,TRUE);
        break;
      case type_du:
        l=new land(mw,land_pxm[type],befahrbarkeit[type],TRUE);
        break;
      default:
        cerr<<"unknown land type\n";
        exit(1);
    }
  } else { cerr<<"type out of range\n"; exit(1); }
  
  return l;
  
}

QPixmap *bildspeicher::get_land_pxm(int type) {
  if ((type>=0) && (type<land_anzahl)) return land_pxm[type];
  else { cerr<<"type out of range\n"; exit(1); }
}

QPixmap *bildspeicher::get_hq_pxm(int spieler) {
  
  int		i;
  
  for (i=0; i<land_anzahl; i++)
    if ((land_type[i]==type_hq) && (land_spieler[i]==spieler)) return land_pxm[i];
  
  cerr<<"cant find hq-pixmap for player "<<spieler<<endl;
  exit(1);
  
}

QPixmap *bildspeicher::get_fa_pxm(int spieler) {
  
  int		i;
  
  for (i=0; i<land_anzahl; i++)
    if ((land_type[i]==type_fa) && (land_spieler[i]==spieler)) return land_pxm[i];
  
  cerr<<"cant find hq-pixmap for player "<<spieler<<endl;
  exit(1);
  
}

QPixmap *bildspeicher::get_de_pxm(int spieler) {
  
  int		i;
  
  for (i=0; i<land_anzahl; i++)
    if ((land_type[i]==type_de) && (land_spieler[i]==spieler)) return land_pxm[i];
  
  cerr<<"cant find hq-pixmap for player "<<spieler<<endl;
  exit(1);
  
}

QPixmap *bildspeicher::get_stellplatz() {
  return stellplatz;
}

void bildspeicher::readStrassenBeschreibung() {
  
  char		fname[512];
  
  strcpy(fname,levelName);
  strcat(fname,"/street_description");
  ifstream	in(fname);
  if (!in) { cerr<<"cant open "<<fname<<endl; exit(1); }
  
  int		i;
  char		name[256];
  
  in>>strasse_anzahl;
  
  strasse=new QPixmap * [strasse_anzahl];
  for (i=0; i<strasse_anzahl; i++) strasse[i]=NULL;
  
  for (i=0; i<strasse_anzahl; i++) {
    in>>name;
    strasse[i]=new QPixmap();
    if (strasse[i]->load(name)==FALSE) { cerr<<"cant load "<<name<<endl; exit(1); }
  }
  
}

void bildspeicher::readEinheitenBeschreibung() {
  
  char		fname[512];
  
  strcpy(fname,levelName);
  strcat(fname,"/unit_description");
  ifstream	in(fname);
  if (!in) { cerr<<"cant open "<<fname<<endl; exit(1); }
  
  int		i;
  
  in>>fahrzeug_anzahl;
  
  fahrzeuge_spieler1=new QPixmap * [fahrzeug_anzahl];
  for (i=0; i<fahrzeug_anzahl; i++) fahrzeuge_spieler1[i]=NULL;
  
  fahrzeuge_spieler2=new QPixmap * [fahrzeug_anzahl];
  for (i=0; i<fahrzeug_anzahl; i++) fahrzeuge_spieler2[i]=NULL;
  
  einheit_beschreibung=new char * [fahrzeug_anzahl];
  for (i=0; i<fahrzeug_anzahl; i++) einheit_beschreibung[i]=NULL;
  
  for (i=0; i<fahrzeug_anzahl; i++) {
    einheit_beschreibung[i]=new char [256];
    fahrzeuge_spieler1[i]=new QPixmap();
    fahrzeuge_spieler2[i]=new QPixmap();
    in>>fname;
    strcpy(einheit_beschreibung[i],levelName);
    strcat(einheit_beschreibung[i],"/");
    strcat(einheit_beschreibung[i],fname);
  }
  
  readFahrzeugePics();
  
}

char *bildspeicher::getEinheitBeschreibung(int type) {
  if ((type>=0) && (type<fahrzeug_anzahl)) return einheit_beschreibung[type];
  else { cerr<<"einheit type out of range\n"; exit(1); }
}

void bildspeicher::readFahrzeugePics() {
  
  ifstream	in;
  int		i;
  char		line[256];
  
  for (i=0; i<fahrzeug_anzahl; i++) {
    in.open(einheit_beschreibung[i]);
    if (!in) { cerr<<"cant open "<<einheit_beschreibung[i]<<endl; exit(1); }
    do {
      in>>line;
    } while ((strcmp(line,"Bild_Spieler1:")!=0) && (in.eof()==FALSE));
    if (in.eof()) { cerr<<"cant find Bild_Spieler1: in "<<einheit_beschreibung[i]<<endl; exit(1); }
    in>>line;
    if (fahrzeuge_spieler1[i]->load(line)==FALSE) { cerr<<"cant open "<<line<<endl; exit(1); }
    in>>line;
    if (strcmp(line,"Bild_Spieler2:")!=0) { cerr<<"cant find Bild_Spieler2: in "<<einheit_beschreibung[i]<<endl; exit(1); }
    in>>line;
    if (fahrzeuge_spieler2[i]->load(line)==FALSE) { cerr<<"cant open "<<line<<endl; exit(1); }
    in.close();
  }
  
}

int bildspeicher::getFahrzeugAnzahl() {
  return fahrzeug_anzahl;
}
