
#include <sys/types.h>
#include <sys/ipc.h>
#include <sys/msg.h>
#include <fstream.h>
#include <stdio.h>
#include <qapplication.h>
#include <unistd.h>

#include "communicator.hh"
#include "einheit.hh"
#include "land.hh"
#include "landschaft.hh"
#include "kampfhandlung.hh"
#include "bewertung.hh"
#include "depot.hh"
#include "menu.hh"

communicator::communicator(main_win *spieler_win) : QTimer () {
  
  MenuMsg	msg;
  
  fertig=FALSE;
  anderer_fertig=FALSE;
  bewegung=NULL;
  angriff=NULL;
  anz_bewegung=0;
  anz_angriff=0;
  anz_uebernahme=0;
  
  mw=spieler_win;
  mw->neuer_communicator(this);
  ls=mw->get_landschaft();
  ls->neuer_communicator(this);
  
  max_bewegung=mw->anz_spieler1+mw->anz_spieler2+5;
  
  bewegung=new koordinaten [max_bewegung];
  
  angriff=new koordinaten [max_bewegung];
  
  ipc_key=ftok(".",'B');
  if (ipc_key==-1) { perror("cant create ipc-key"); exit(1); }
  
  msq_id=msgget(ipc_key,IPC_CREAT|0660);
  if (msq_id==-1) { perror("cant create msg-queue"); exit(1); }
  
  msg.mtype=MSGTYPE_PID;
  msg.pid=getpid();
  if (msgsnd(msq_id,(struct msgbuf *)(&msg),sizeof(MenuMsg)-sizeof(long),0)==-1) { perror("cant send message, deadlock: start_game waites for answer"); exit(1); }
  
  sp=mw->bin_spieler;
  if (sp==1) asp=2;
  else asp=1;
  
  connect(this,SIGNAL(timeout()),this,SLOT(readMessageFertig()));
  
  start(250);
  
}

communicator::~communicator() {
  
  if (bewegung!=NULL) delete bewegung;
  if (angriff!=NULL) delete angriff;
  
}

void communicator::modus_wechseln() {
  
  if (fertig && anderer_fertig) {
    
    if (mw->aktueller_modus()==angriffsmodus) {
      // Angriff durchfuehren
      angriff_durchfuehren();
      // Bewegungen empfangen
      bewegungen_empfangen();
      // Werte der Einheiten senden
      einheiten_senden();
      // Depotenergien senden
      sende_depot_energie();
      // Depotenergien empfangen
      empfange_depot_energie();
      // Depot Eroberung empfangen
      uebernahme_empfangen();
    } else {
      // Bewegungen senden
      bewegungen_senden();
      // Werte der Einheiten empfangen
      einheiten_empfangen();
      // Depotenergien senden
      sende_depot_energie();
      // Depotenergien empfangen
      empfange_depot_energie();
      //Depot Eroberung senden
      uebernahme_senden();
    }
    
    anz_bewegung=0;
    anz_angriff=0;
    anz_uebernahme=0;
    
    mw->modus_wechseln();
    fertig=FALSE;
    anderer_fertig=FALSE;
    
  }
  
}

void communicator::bereit_fuer_moduswechsel(int bo) {
  
  BattleMsg	msg;
  
  fertig=bo;
  msg.mtype=sp+BEFEHL_FERTIG;
  msg.fertig=fertig;
  if (msgsnd(msq_id,(msgbuf *)(&msg),sizeof(BattleMsg)-sizeof(long),0)==-1) { perror("cant send message"); exit(1); }
  
}

void communicator::melde_bewegung(int sx, int sy, int ex, int ey, int platz_start, int platz_ziel) {
  
  bewegung[anz_bewegung].sx=sx;
  bewegung[anz_bewegung].sy=sy;
  bewegung[anz_bewegung].ex=ex;
  bewegung[anz_bewegung].ey=ey;
  bewegung[anz_bewegung].platz_start=platz_start;
  bewegung[anz_bewegung++].platz_ziel=platz_ziel;
  
  if (anz_bewegung>=max_bewegung) { cerr<<"zuviele Bewegungen!\n"; exit(1); } 
  
}

void communicator::melde_angriff(int sx, int sy, int ex, int ey, int gegenwehr) {
  
  angriff[anz_angriff].sx=sx;
  angriff[anz_angriff].sy=sy;
  angriff[anz_angriff].ex=ex;
  angriff[anz_angriff].ey=ey;
  angriff[anz_angriff++].gegenwehr=gegenwehr;
  
  if (anz_angriff>=mw->anz_spieler1+mw->anz_spieler2) { cerr<<"zuviele Angriffe!\n"; exit(1); }
  
}

void communicator::angriff_durchfuehren() {
  
  if (mw->aktueller_modus()==bewegungsmodus) {
    if (anz_angriff>0) {
      cerr<<"communicator::angriff_durchfuehren() interner Fehler 1\n";
      exit(1);
    }
    return;
  }
  
  einheit	*e_von, *e_nach;
  int		i, old_anz1, old_anz2;
  KampfMsg	msg;
  
  for (i=0; i<anz_angriff; i++) {
    
    e_von=ls->get_einheit(angriff[i].sx,angriff[i].sy);
    e_nach=ls->get_einheit(angriff[i].ex,angriff[i].ey);
    
    if (e_von==NULL) { cerr<<"e_von==NULL interner Fehler!\n"; exit(1); }
    if (e_nach!=NULL) {
      
      old_anz1=e_von->anzahl;
      old_anz2=e_nach->anzahl;
      
      bewertung(e_von,e_nach,angriff[i].gegenwehr);
      
      msg.ang_old_anz=old_anz1;
      msg.geg_old_anz=old_anz2;
      msg.ang_anz=e_von->anzahl;
      msg.geg_anz=e_nach->anzahl;
      msg.gegenwehr=angriff[i].gegenwehr;
      strncpy(msg.ang_pxm_name,e_von->pxm_name,64);
      strncpy(msg.geg_pxm_name,e_nach->pxm_name,64);
      strncpy(msg.ang_msl_name,e_von->getMissleName(),64);
      strncpy(msg.geg_msl_name,e_nach->getMissleName(),64);
      
      msg.mtype=BEFEHL_KAMPF_START+sp;
      if (msgsnd(msq_id,(msgbuf *)(&msg),sizeof(KampfMsg)-sizeof(long),0)==-1) { perror("cant send message"); exit(1); }
      msg.mtype=BEFEHL_KAMPF_START+asp;
      if (msgsnd(msq_id,(msgbuf *)(&msg),sizeof(KampfMsg)-sizeof(long),0)==-1) { perror("cant send message"); exit(1); }
      
      if (msgrcv(msq_id,(msgbuf *)(&msg),sizeof(KampfMsg)-sizeof(long),BEFEHL_KAMPF_FERTIG+sp,0)==-1) { perror("cant recieve message"); exit(1); }
      if (msgrcv(msq_id,(msgbuf *)(&msg),sizeof(KampfMsg)-sizeof(long),BEFEHL_KAMPF_FERTIG+asp,0)==-1) { perror("cant recieve message"); exit(1); }
      
      if (e_von->anzahl<=0) {
        ls->einheit_zerstoert(angriff[i].sx,angriff[i].sy);
        e_nach->ausbildung+=2;
      } else e_nach->ausbildung+=1;
      
      if (e_nach->anzahl<=0) {
        ls->einheit_zerstoert(angriff[i].ex,angriff[i].ey);
        e_von->ausbildung+=2;
      } else e_von->ausbildung+=1;
      
      if (e_von->ausbildung>10) e_von->ausbildung=10;
      if (e_nach->ausbildung>10) e_nach->ausbildung=10;
      
    }
    
  }
  
}

void communicator::bewegungen_senden() {
  
  int		i;
  BattleMsg	msg;
  
  msg.mtype=sp+BEFEHL_BEWEGUNG_START;
  msg.anzahl=anz_bewegung;
  if (msgsnd(msq_id,(msgbuf *)(&msg),sizeof(BattleMsg)-sizeof(long),0)==-1) { perror("cant send message"); exit(1); }
  
  for (i=0; i<anz_bewegung; i++) {
    msg.mtype=sp+BEFEHL_BEWEGUNG;
    msg.k=bewegung[i];
    if (msgsnd(msq_id,(msgbuf *)(&msg),sizeof(BattleMsg)-sizeof(long),0)==-1) { perror("cant send message"); exit(1); }
  }
  
}

void communicator::bewegungen_empfangen() {
  
  int		i;
  BattleMsg	msg;
  
  if (msgrcv(msq_id,(msgbuf *)(&msg),sizeof(BattleMsg)-sizeof(long),asp+BEFEHL_BEWEGUNG_START,0)==-1) { perror("cant recieve message"); exit(1); }
  anz_bewegung=msg.anzahl;
  
  for (i=0; i<anz_bewegung; i++) {
    if (msgrcv(msq_id,(msgbuf *)(&msg),sizeof(BattleMsg)-sizeof(long),asp+BEFEHL_BEWEGUNG,0)==-1) { perror("cant recieve message"); exit(1); }
    ls->bewegung_ausfuehren(msg.k.sx,msg.k.sy,msg.k.ex,msg.k.ey,msg.k.platz_start,msg.k.platz_ziel);
  }
  
}

void communicator::einheiten_senden() {
  
  int		i, j, k, w, h, anz;
  BattleMsg	msg;
  land		*l;
  einheit	*e;
  
  ls->landschaft_groesse(&w,&h);
  
  anz=0;
  for (i=0; i<w; i++) {
    for (j=0; j<h; j++) {
      l=ls->get_land(i,j);
      if (l->bin_depot()) {
        for (k=0; k<anz_inhalt; k++) {
          e=l->get_einheit(k);
          if (e!=NULL) {
            msg.k.sx=i;
            msg.k.sy=j;
            msg.k.platz_start=k;
            e->werte2data(msg.e);
            msg.mtype=sp+BEFEHL_EINHEIT;
            if (msgsnd(msq_id,(msgbuf *)(&msg),sizeof(BattleMsg)-sizeof(long),0)==-1) { perror("cant send message"); exit(1); }
            anz++;
          }
        }
      } else {
        e=l->get_einheit();
        if (e!=NULL) {
          msg.k.sx=i;
          msg.k.sy=j;
          msg.k.platz_start=0;
          e->werte2data(msg.e);
          msg.mtype=sp+BEFEHL_EINHEIT;
          if (msgsnd(msq_id,(msgbuf *)(&msg),sizeof(BattleMsg)-sizeof(long),0)==-1) { perror("cant send message"); exit(1); }
          anz++;
        }
      }
    }
  }
  
  msg.mtype=sp+BEFEHL_EINHEIT_START;
  msg.anzahl=anz;
  if (msgsnd(msq_id,(msgbuf *)(&msg),sizeof(BattleMsg)-sizeof(long),0)==-1) { perror("cant send message"); exit(1); }
  
}

void communicator::einheiten_empfangen() {
  
  int		i, j, k, w, h, anz;
  BattleMsg	msg;
  land		*l;
  einheit	*e;
  
  if (msgrcv(msq_id,(msgbuf *)(&msg),sizeof(BattleMsg)-sizeof(long),asp+BEFEHL_EINHEIT_START,0)==-1) { perror("cant recieved message"); exit(1); }
  anz=msg.anzahl;
  
  ls->landschaft_groesse(&w,&h);
  
  int		feld[w][h][anz_inhalt];
  
  for (i=0; i<w; i++) for (j=0; j<h; j++) for (k=0; k<anz_inhalt; k++)feld[i][j][k]=FALSE;
  
  for (i=0; i<anz; i++) {
    if (msgrcv(msq_id,(msgbuf *)(&msg),sizeof(BattleMsg)-sizeof(long),asp+BEFEHL_EINHEIT,0)==-1) { perror("cant recieved message"); exit(1); }
    e=ls->get_einheit(msg.k.sx,msg.k.sy,msg.k.platz_start);
    l=ls->get_land(msg.k.sx,msg.k.sy);
    if (l->bin_depot()==FALSE) {
      if (e!=NULL) {
        e->data2werte(msg.e);
        feld[msg.k.sx][msg.k.sy][msg.k.platz_start]=TRUE;
      } else {
        cerr<<"Fehler Einheiten abgleichen: Platz ";
        cerr<<msg.k.sx<<" "<<msg.k.sy<<" "<<msg.k.platz_start<<endl;
        exit(1);
      }
    } else {
      if (e!=NULL) {
        e->data2werte(msg.e);
        feld[msg.k.sx][msg.k.sy][msg.k.platz_start]=TRUE;
      } else {
        e=new einheit(mw,msg.e.type,msg.e.gehoere_zu);
        e->data2werte(msg.e);
        l->einheit_kommt_an(e,msg.k.platz_start);
        feld[msg.k.sx][msg.k.sy][msg.k.platz_start]=TRUE;
      }
    }
  }
  
  // nach zerstoerten Einheiten suchen
  
  for (i=0; i<w; i++) {
    for (j=0; j<h; j++) {
      l=ls->get_land(i,j);
      if (l->bin_depot()) {
        for (k=0; k<anz_inhalt; k++) {
          if (feld[i][j][k]==FALSE) {
            e=l->get_einheit(k);
            if (e!=NULL) l->einheit_zerstoert(k);
          }
        }
      } else {
        if (feld[i][j][0]==FALSE) {
          e=l->get_einheit();
          if (e!=NULL) l->einheit_zerstoert();
        }
      }
    }
  }
  
}

int communicator::warten_auf_wechsel() {
  return fertig;
}

void communicator::melde_uebernahme(int x, int y) {
  
  uebernahme[anz_uebernahme].sx=x;
  uebernahme[anz_uebernahme++].sy=y;
  
  if (anz_uebernahme>=max_uebernahme) { cerr<<"zuviele Depot-Eroberungen\n"; exit(1); }
  
}

void communicator::uebernahme_senden() {
  
  int		i, j, anz=0;
  einheit	*e;
  land		*l;
  BattleMsg	msg;
  
  for (i=0; i<anz_uebernahme; i++) {
    // im Depot pruefen, ob Fusstruppe erfolgreich eingetrungen
    l=ls->get_land(uebernahme[i].sx,uebernahme[i].sy);
    if (l->bin_depot()==FALSE) { cerr<<"uebernahme von land nicht moeglich\n"; exit(1); }
    for (j=0; j<anz_inhalt; j++) {
      e=l->get_einheit(j);
      if (e!=NULL) {
        if ((e->gehoere_zu!=l->gehoere_zu()) && (e->fusstruppe())) {
          // Depot uebernehmen
          j=anz_inhalt;
          l->uebernahmeDurchSpieler(e->gehoere_zu);
          msg.mtype=sp+BEFEHL_UEBERNAHME;
          msg.k=uebernahme[i];
          msg.spieler=e->gehoere_zu;
          if (msgsnd(msq_id,(msgbuf *)(&msg),sizeof(BattleMsg)-sizeof(long),0)==-1) { perror("cant send message"); exit(1); }
          anz++;
        }
      }
    }
  }
  
  msg.mtype=sp+BEFEHL_UEBERNAHME_START;
  msg.anzahl=anz;
  if (msgsnd(msq_id,(msgbuf *)(&msg),sizeof(BattleMsg)-sizeof(long),0)==-1) { perror("cant send message"); exit(1); }
  
}

void communicator::uebernahme_empfangen() {
  
  int		i, anz;
  land		*l;
  BattleMsg	msg;
  
  if (msgrcv(msq_id,(msgbuf *)(&msg),sizeof(BattleMsg)-sizeof(long),asp+BEFEHL_UEBERNAHME_START,0)==-1) { perror("cant recieved message"); exit(1); }
  anz=msg.anzahl;
  
  for (i=0; i<anz; i++) {
    if (msgrcv(msq_id,(msgbuf *)(&msg),sizeof(BattleMsg)-sizeof(long),asp+BEFEHL_UEBERNAHME,0)==-1) { perror("cant recieved message"); exit(1); }
    l=ls->get_land(msg.k.sx,msg.k.sy);
    if (l->bin_depot()==FALSE) { cerr<<"uebernahme von land nicht moeglich; interner Fehler\n"; exit(1); }
    // Depot uebernehmen
    l->uebernahmeDurchSpieler(msg.spieler);
  }
  
}

void communicator::readMessageFertig() {
  
  BattleMsg	msg;
  int		r;
  
  do {
    r=msgrcv(msq_id,(msgbuf *)(&msg),sizeof(BattleMsg)-sizeof(long),asp+BEFEHL_EXIT,IPC_NOWAIT);
    if (r!=-1) {
      msg.mtype=sp+BEFEHL_EXIT;
      if (msgsnd(msq_id,(msgbuf *)(&msg),sizeof(BattleMsg)-sizeof(long),0)==-1) { perror("cant send message"); exit(1); }
      exit(0);
    }
  } while (r!=-1);
  
  do {
    r=msgrcv(msq_id,(msgbuf *)(&msg),sizeof(BattleMsg)-sizeof(long),asp+BEFEHL_FERTIG,IPC_NOWAIT);
    if (r!=-1) anderer_fertig=msg.fertig;
  } while (r!=-1);
  
  if (fertig && anderer_fertig) modus_wechseln();
  
}

void communicator::sendKampfbildschirmMsg(int status) {
  
  KampfMsg	msg;
  
  msg.mtype=BEFEHL_KAMPFBILDSCHIRM+sp;
  if (status==1) msg.kampfbildschirm=TRUE;
  else msg.kampfbildschirm=FALSE;
  
  if (msgsnd(msq_id,(msgbuf *)(&msg),sizeof(KampfMsg)-sizeof(long),0)==-1) { perror("cant send message"); exit(1); }
  
}

void communicator::victory(char *txt) {
  
  MenuMsg	msg;
  
  msg.mtype=MSGTYPE_VICTORY;
  strncpy(msg.victory,txt,VICTORY_TXT_SIZE);
  
  if (msgsnd(msq_id,(msgbuf *)(&msg),sizeof(MenuMsg)-sizeof(long),0)==-1) { perror("cant send message"); exit(1); }
  
  exit(0);
  
}

void communicator::sende_depot_energie() {
  
  DepotMsg	msg;
  int		x, y, w, h;
  land		*l;
  
  ls->landschaft_groesse(&w,&h);
  
  for (x=0; x<w; x++) {
    for (y=0; y<h; y++) {
      l=ls->get_land(x,y);
      if ((l->bin_depot()==TRUE) && (l->gehoere_zu()==sp)) {
        msg.mtype=sp+BEFEHL_DEPOT;
        msg.x=x;
        msg.y=y;
        msg.e=l->wieviel_energie();
        if (msgsnd(msq_id,(msgbuf *)(&msg),sizeof(DepotMsg)-sizeof(long),0)==-1) { perror("cant send message"); exit(1); }
      }
    }
  }
  
}

void communicator::empfange_depot_energie() {
  
  DepotMsg	msg;
  int		anz_dep, x, y, w, h;
  land		*l;
  
  anz_dep=0;
  ls->landschaft_groesse(&w,&h);
  for (x=0; x<w; x++) {
    for (y=0; y<h; y++) {
      l=ls->get_land(x,y);
      if ((l->bin_depot()==TRUE) && (l->gehoere_zu()==asp)) anz_dep++;
    }
  }
  
  for (x=0; x<anz_dep; x++) {
    if (msgrcv(msq_id,(msgbuf *)(&msg),sizeof(DepotMsg)-sizeof(long),BEFEHL_DEPOT+asp,0)==-1) { perror("cant recieve message"); exit(1); }
    l=ls->get_land(msg.x,msg.y);
    l->setEnergy(msg.e);
  }
  
}

