
#ifndef communicator_header
#define communicator_header

#include <qtimer.h>

#include <sys/types.h>
#include <sys/ipc.h>

#include "main_win.hh"
#include "landschaft.hh"

#define max_uebernahme		10

#define BEFEHL_FERTIG		0x10
#define BEFEHL_EINHEIT		0x20
#define BEFEHL_EINHEIT_START	0x40
#define BEFEHL_BEWEGUNG		0x80
#define BEFEHL_BEWEGUNG_START	0x100
#define BEFEHL_UEBERNAHME	0x200
#define BEFEHL_UEBERNAHME_START	0x400
#define BEFEHL_KAMPF_START	0x800
#define BEFEHL_KAMPF_FERTIG	0x1000
#define BEFEHL_KAMPFBILDSCHIRM	0x2000
#define BEFEHL_EXIT		0x4000
#define BEFEHL_DEPOT		0x8000

typedef struct {
  int sx, sy, ex, ey, platz_start, platz_ziel, gegenwehr;
} koordinaten;

typedef struct {
  long		mtype;
  koordinaten	k;
  int		anzahl, fertig, spieler;
  einheitData	e;
} BattleMsg;

typedef struct {
  long		mtype;
  int		x, y, e;
} DepotMsg;

class communicator : QTimer {
  
  Q_OBJECT
  
  public:
  
  communicator(main_win *spieler_win);
  ~communicator();
  
  void bereit_fuer_moduswechsel(int bo);
  int warten_auf_wechsel();
  
  void melde_bewegung(int sx, int sy, int ex, int ey, int platz_start, int platz_ziel);
  void melde_angriff(int sx, int sy, int ex, int ey, int gegenwehr);
  void melde_uebernahme(int x, int y);
  
  void sendKampfbildschirmMsg(int status);
  
  void victory(char *txt);
  
  private slots:
  
  void readMessageFertig();
  
  private:
  
  main_win	*mw;
  landschaft	*ls;
  int		fertig, anderer_fertig,
  		anz_bewegung, anz_angriff, anz_uebernahme, max_bewegung;
  koordinaten	*bewegung, *angriff, uebernahme[max_uebernahme];
  
  key_t		ipc_key;
  int		msq_id, sp, asp;
  
  void modus_wechseln();
  void angriff_durchfuehren();
  
  void einheiten_senden();
  void einheiten_empfangen();
  void bewegungen_senden();
  void bewegungen_empfangen();
  void uebernahme_senden();
  void uebernahme_empfangen();
  void sende_depot_energie();
  void empfange_depot_energie();
  
};

#endif

