
#include <unistd.h>
#include <fstream.h>

#include "computerspieler.hh"
#include "land.hh"
#include "depot.hh"

#define MAX_DEPOT	10
#define REPAIR_CHECK	4

computerspieler::computerspieler() {
  
  mw=NULL;
  ls=NULL;
  fahrzeug_anzahl=0;
  
}

void computerspieler::init(main_win *main_w) {
  
  int		i;
  
  fz=NULL;
  fe=NULL;
  
  mw=main_w;
  ls=mw->get_landschaft();
  
  fahrzeug_anzahl=mw->bs.getFahrzeugAnzahl();
  
  if (mw->bin_spieler==1) {
    max_fz=mw->anz_spieler1+20;
    max_fe=mw->anz_spieler2+20;
  } else {
    max_fz=mw->anz_spieler2+20;
    max_fe=mw->anz_spieler1+20;
  }
  fz=new Fahrzeug [max_fz];
  fe=new Fahrzeug [max_fe];
  
  for (i=0; i<max_fz; i++) fz[i].e=NULL;
  for (i=0; i<max_fe; i++) fe[i].e=NULL;
  
}

computerspieler::~computerspieler() {
  
  if (fz!=NULL) delete fz;
  if (fe!=NULL) delete fe;
  
}

void computerspieler::swap(int & var1, int & var2) {
  
  int		s;
  
  s=var1;
  var1=var2;
  var2=s;
  
}

void computerspieler::swap(Fahrzeug & var1, Fahrzeug & var2) {
  
  Fahrzeug	s;
  
  s=var1;
  var1=var2;
  var2=s;
  
}

int computerspieler::zaehle_feinde() {
  
  int		x, y, w, h, f;
  land		*la;
  einheit	*e;
  
  // zaehlt die Feinde, die nicht im Depot sind
  
  ls->landschaft_groesse(&w,&h);
  
  f=0;
  for (x=0; x<w; x++) {
    for (y=0; y<h; y++) {
      la=ls->get_land(x,y);
      if (la->bin_depot()==FALSE) {
        e=la->get_einheit();
        if (e!=NULL)
          if (e->gehoere_zu!=mw->bin_spieler) f++;
      }
    }
  }
  
  return f;
  
}

void computerspieler::fahrzeuge_aktualisieren() {
  
  int		lw, lh, x, y, p, pp, i;
  land		*la;
  einheit	*e;
  
  // erstellt eine Liste der eigenen und der feindlichen Fahrzeuge
  
  ls->landschaft_groesse(&lw,&lh);
  
  p=0;
  pp=0;
  for (x=0; x<lw; x++) {
    for (y=0; y<lh; y++) {
      la=ls->get_land(x,y);
      if (la->bin_depot()==FALSE) {
        e=la->get_einheit();
        if (e!=NULL) {
          if (e->gehoere_zu==mw->bin_spieler) {
            if (p>=max_fz) { cerr<<"computerspieler::fahrzeuge_aktualisieren() interner Fehler!\n"; exit(1); }
            fz[p].x=x;
            fz[p].y=y;
            fz[p++].e=e;
          } else {
            if (pp>=max_fe) { cerr<<"computerspieler::fahrzeuge_aktualisieren() interner Fehler!\n"; exit(1); }
            fe[pp].x=x;
            fe[pp].y=y;
            fe[pp++].e=e;
          }
        }
      }
    }
  }
  for (i=p; i<max_fz; i++) fz[i].e=NULL;
  for (i=pp; i<max_fe; i++) fe[i].e=NULL;
  
}

void computerspieler::bewegen() {
  
  int		mx, my, i, j, k, anz, bo, abst1, abst2, g, lw, lh,
  		ziel_x=0, ziel_y=0, x, y, b,
		ziel_feld_x, ziel_feld_y, var;
  land		*la;
  einheit	*ei, *nb;
  
  fahrzeuge_aktualisieren();
  ls->landschaft_groesse(&lw,&lh);
  
  // ### Rudeltaktik ###
  
  // 1. Mittelpunkt der Einheiten berechnen
  
  anz=0;
  mx=0;
  my=0;
  for (i=0; i<max_fz; i++) {
    if (fz[i].e!=NULL) {
      anz++;
      mx+=fz[i].x;
      my+=fz[i].y;
    }
  }
  if (anz>0) {
    mx/=anz;
    my/=anz;
  }
  
  // Feinde nach Entfernung sortieren
  
  // Bubblesort
  anz=zaehle_feinde();
  bo=TRUE;
  while (bo) {
    bo=FALSE;
    for (i=1; i<anz; i++) {
      abst1=abstand(fe[i-1].x,fe[i-1].y,mx,my);
      abst2=abstand(fe[i].x,fe[i].y,mx,my);
      if (abst1>abst2) {
        bo=TRUE;
        swap(fe[i-1],fe[i]);
      }
    }
  }
  
  // auf 3 der naechsten Feinde zubewegen in 3 Gruppen
  g=0;
  for (i=0; i<max_fz; i++) {
    if (fz[i].e!=NULL) {
      ls->steuerung_bewegungsmodus(fz[i].x,fz[i].y);
      ziel_feld_x=-1;
      if (fz[i].e->anzahl<=REPAIR_CHECK) naechstesDepot(ziel_feld_x,ziel_feld_y,fz[i].x,fz[i].y);
      if (ziel_feld_x==-1) {
        // ran an den Feind
        ziel_feld_x=fe[g].x;
        ziel_feld_y=fe[g].y;
      }
      ziel_x=ziel_feld_x;
      ziel_y=ziel_feld_y;
      abst1=1000000000;
      // Feld mit geringstem Abstand zum Ziel-Feld suchen
      for (x=0; x<lw; x++) {
        for (y=0; y<lh; y++) {
          la=ls->get_land(x,y);
          if (la->rahmen()) {
            abst2=abstand(x,y,ziel_feld_x,ziel_feld_y);
            if (abst2<abst1) {
              if (fz[i].e->max_schussweite>1) {
                // Einheit ist Artillerie
                var=TRUE;
                nb=ls->nachbar(x,y,nachbar_o);
                if (nb!=NULL) if (nb->gehoere_zu!=fz[i].e->gehoere_zu) var=FALSE;
                nb=ls->nachbar(x,y,nachbar_ol);
                if (nb!=NULL) if (nb->gehoere_zu!=fz[i].e->gehoere_zu) var=FALSE;
                nb=ls->nachbar(x,y,nachbar_or);
                if (nb!=NULL) if (nb->gehoere_zu!=fz[i].e->gehoere_zu) var=FALSE;
                nb=ls->nachbar(x,y,nachbar_u);
                if (nb!=NULL) if (nb->gehoere_zu!=fz[i].e->gehoere_zu) var=FALSE;
                nb=ls->nachbar(x,y,nachbar_ul);
                if (nb!=NULL) if (nb->gehoere_zu!=fz[i].e->gehoere_zu) var=FALSE;
                nb=ls->nachbar(x,y,nachbar_ur);
                if (nb!=NULL) if (nb->gehoere_zu!=fz[i].e->gehoere_zu) var=FALSE;
                if (var==TRUE) {
                  ziel_x=x;
                  ziel_y=y;
                  abst1=abst2;
                }
              } else {
                ziel_x=x;
                ziel_y=y;
                abst1=abst2;
              }
            }
          }
        }
      }
      ls->steuerung_bewegungsmodus(ziel_x,ziel_y);
      if (g++==3) g=0;
    }
  }
  
  // reparierte Fahrzeuge aus Depot bewegen
  
  for (i=0; i<lw; i++) {
    for (j=0; j<lh; j++) {
      la=ls->get_land(i,j);
      if (la->bin_depot()) {
        for (k=0; k<anz_inhalt; k++) {
          ei=la->get_einheit(k);
          if (ei!=NULL) {
            la->set_akt_stellplatz(k);
            if (la->wieviel_energie()<3) {
              // alle Fahrzeuge rausbewegen
              ls->steuerung_bewegungsmodus(i,j);
              b=FALSE;
              for (x=0; x<lw; x++) {
                for (y=0; y<lh; y++) {
                  la=ls->get_land(x,y);
                  if (la->rahmen()) {
                    b=TRUE;
                    ls->steuerung_bewegungsmodus(x,y);
                  }
                }
              }
              if (b==FALSE) ls->steuerung_reset();
            } else {
              // nur die reparierten rausbewegen
              if (ei->anzahl==10) {
                ls->steuerung_bewegungsmodus(i,j);
                b=FALSE;
                for (x=0; x<lw; x++) {
                  for (y=0; y<lh; y++) {
                    la=ls->get_land(x,y);
                    if (la->rahmen()) {
                      b=TRUE;
                      ls->steuerung_bewegungsmodus(x,y);
                    }
                  }
                }
                if (b==FALSE) ls->steuerung_reset();
              }
            }
          }
        }
      }
    }
  }
  
  mw->modus_change_pressed();
  ls->paint_new();
  
}

void computerspieler::angreifen() {
  
  int		i, j, k, lw, lh, x, y, anz, bo, v1, v2;
  land		*la;
  einheit	*ei;
  Fahrzeug	ziele[zaehle_feinde()];
  
  fahrzeuge_aktualisieren();
  
  ls->landschaft_groesse(&lw,&lh);
  
  for (i=0; i<max_fz; i++) {
    
    // moegliche Ziele finden
    anz=0;
    if (fz[i].e!=NULL) {
      if ((fz[i].e->gehoere_zu==mw->bin_spieler) && (fz[i].e->anzahl>REPAIR_CHECK)) {
        ls->steuerung_angriffsmodus(fz[i].x,fz[i].y);
        for (x=0; x<lw; x++) {
          for (y=0; y<lh; y++) {
            la=ls->get_land(x,y);
            if (la->rahmen()) {
              ziele[anz].x=x;
              ziele[anz].y=y;
              ziele[anz++].e=la->get_einheit();
            }
          }
        }
      }
    }
    if (anz!=0) {
      // bestes Ziel auswaehlen
      // Ziele sortieren nach:
      // 1. Feuerkraft
      // 2. Ausbildung
      // 3. Anzahl
      
      // Bubblesort
      bo=TRUE;
      while (bo) {
        bo=FALSE;
        for (j=1; j<anz; j++) {
          // 1. Feuerkraft
          if (fz[i].e->flugzeug) {
            v1=ziele[j-1].e->feuerkraft_luft/10;
            v2=ziele[j].e->feuerkraft_luft/10;
          } else {
            v1=ziele[j-1].e->feuerkraft_boden/10;
            v2=ziele[j].e->feuerkraft_boden/10;
          }
          // 2. Ausbildung & 3. Anzahl
          v1+=ziele[j-1].e->ausbildung+ziele[j-1].e->anzahl;
          v2+=ziele[j].e->ausbildung+ziele[j].e->anzahl;
          if (v1>v2) {
            bo=TRUE;
            swap(ziele[j-1],ziele[j]);
          }
        }
      }
      
      // Ziel nach Vorgabe von Einheit auswaehlen
      for (k=0; k<fahrzeug_anzahl; k++) {
        for (j=0; j<anz; j++) {
          if (fz[i].e->StrategieDaten[k]==ziele[j].e->was_bin_ich)
            ls->steuerung_angriffsmodus(ziele[j].x,ziele[j].y);
        }
      }
      
    }
    
  }
  
  // Fahrzeuge reparieren
  
  for (i=0; i<lw; i++) {
    for (j=0; j<lh; j++) {
      la=ls->get_land(i,j);
      if (la->bin_depot()) {
        if (la->gehoere_zu()==mw->bin_spieler) {
          for (k=0; k<anz_inhalt; k++) {
            ei=la->get_einheit(k);
            if (ei!=NULL) la->repariere_einheit(k);
          }
        }
      }
    }
  }
  
  mw->modus_change_pressed();
  ls->paint_new();
  
}

int computerspieler::abstand(int x1, int y1, int x2, int y2) {
  return (x1-x2)*(x1-x2)+(y1-y2)*(y1-y2);
}

void computerspieler::naechstesDepot(int & x, int & y, int posx, int posy) {
  
  int		w, h, i, j, p,
		dep_abst[MAX_DEPOT],
		dep_x[MAX_DEPOT],
		dep_y[MAX_DEPOT];
  land		*l;
  
  ls->landschaft_groesse(&w,&h);
  
  p=0;
  for (i=0; i<w; i++) {
    for (j=0; j<h; j++) {
      l=ls->get_land(i,j);
      if ((l->gehoere_zu()==mw->bin_spieler) && (l->bin_depot()==TRUE) && (l->wieviel_energie()>=3)) {
        dep_x[p]=i;
        dep_y[p]=j;
        dep_abst[p++]=abstand(i,j,posx,posy);
        if (p>=MAX_DEPOT) { cerr<<"computerspieler: MAX_DEPOT zu klein\n"; exit(1); }
      }
    }
  }
  
  if (p==0) {
    x=-1;
    y=-1;
    return;
  }
  
  // sortieren
  
  i=TRUE;
  while (i) {
    i=FALSE;
    for (j=1; j<p; j++) {
      if (dep_abst[j-1]>dep_abst[j]) {
        i=TRUE;
        swap(dep_abst[j],dep_abst[j-1]);
        swap(dep_x[j],dep_x[j-1]);
        swap(dep_y[j],dep_y[j-1]);
      }
    }
  }
  
  x=dep_x[0];
  y=dep_y[0];
  
}

