
#include <fstream.h>
#include <stdio.h>
#include <unistd.h>
#include <qbrush.h>
#include <math.h>
#include <qcolor.h>

#include "landschaft.hh"
#include "main_win.hh"
#include "communicator.hh"
#include "einheit.hh"
#include "depot.hh"

#define versch_x	44
#define versch_y	30

#define karte_rect_size	6

extern int einheit_id_count;

landschaft::landschaft(char *filename, QWidget *parent, const char *name)
  : QWidget(parent,name) {
  
  mw = (main_win *) (topLevelWidget());
  co=NULL;
  karte=NULL;
  
  resize(300,200);
  
  offx=1;
  offy=1;
  
  klick=1;
  
  paint=NULL;
  la=NULL;
  updown=NULL;
  leftright=NULL;
  
  paint=new QPainter();
  
  read(filename);
  
  updown=new QScrollBar(1,1000,50,250,1,QScrollBar::Vertical,parent,"updown");
  updown->setTracking(TRUE);
  
  leftright=new QScrollBar(1,1000,50,250,1,QScrollBar::Horizontal,parent,"leftright");
  leftright->setTracking(TRUE);
  
  connect(updown,SIGNAL(valueChanged(int)),this,SLOT(set_offy(int)));
  connect(leftright,SIGNAL(valueChanged(int)),this,SLOT(set_offx(int)));
  connect(updown,SIGNAL(sliderReleased()),this,SLOT(paint_new()));
  connect(leftright,SIGNAL(sliderReleased()),this,SLOT(paint_new()));
  
  if (mw->bin_spieler==1) anderer_spieler=2;
  else anderer_spieler=1;
  
  karte=new QPixmap(landwidth*karte_rect_size,landheight*karte_rect_size+karte_rect_size/2);
  
}

landschaft::~landschaft() {
  
  int		i, j;
  
  if (karte!=NULL) delete karte;
  if (paint!=NULL) delete paint;
  if (updown!=NULL) delete updown;
  if (leftright!=NULL) delete leftright;
  if (la!=NULL) {
    for (i=0; i<landwidth; i++) {
      if (la[i]!=NULL) {
        for (j=0; j<landheight; j++) if ( (la[i])[j]!=NULL ) delete (la[i])[j];
        delete la[i];
      }
    }
    delete la;
  }
  
}

void landschaft::paintEvent(QPaintEvent *) {
  
  int	i, j, posx, posy;
  
  updown->move(x()+width()+10,y());
  leftright->move(x(),y()+height()+10);
  updown->resize(20,height());
  leftright->resize(width(),20);
  
  j=0;
  posy=offy;
  paint->begin(this);
  for (j=0; j<landheight; j++) {
    posx=offx;
    for (i=0; i<landwidth; i+=2) {
      if ( (posx+versch_x*3/2>=0) && (posx<width()) && (posy+versch_y*2>=0) && (posy<height()) )
        (la[i])[j]->draw_ansicht(paint,posx,posy);
      posx+=versch_x*2;
    }
    posx=offx+versch_x;
    for (i=1; i<landwidth; i+=2) {
      if ( (posx+versch_x*3/2>=0) && (posx<width()) && (posy+versch_y*3>=0) && (posy+versch_y<height()) )
        (la[i])[j]->draw_ansicht(paint,posx,posy+versch_y);
      posx+=versch_x*2;
    }
    posy+=versch_y*2;
  }
  paint->end();
  
}

int landschaft::read(char *filename) {
  
  ifstream	eingabe(filename);
  char		line[256];
  int		i, j, typ;
  
  einheit_id_count=0;
  
  if (la!=NULL) { cerr<<"landschaft::read(..) darf nur vom Konstruktor aufgerufen werden!\n"; exit(1); }
  
  if (!eingabe) { cerr<<"Level-Datei "<<filename<<" konnte nicht geoeffnet werden!\n"; exit(1); }
  
  eingabe>>line;
  if (strcmp("XB.I.C.-Level",line)!=0) { cerr<<"Falsches Dateiformat von "<<filename<<" !\n"; exit(1); }
  
  eingabe>>line;
  if (strcmp("Groesse:",line)!=0) { cerr<<"Falsches Dateiformat von "<<filename<<" !\n"; exit(1); }
  eingabe>>line;
  sscanf(line,"%d",&landwidth);
  eingabe>>line;
  sscanf(line,"%d",&landheight);
  
  eingabe>>line;
  if (strcmp("Landschaft:",line)!=0) { cerr<<"Falsches Dateiformat von "<<filename<<" !\n"; exit(1); }
  la=new land ** [landwidth];
  if (la==NULL) { cerr<<"Zuwenig Speicher!\n"; exit(1); }
  for (i=0; i<landwidth; i++) la[i]=NULL;
  
  for (i=0; i<landwidth; i++) {
    la[i]=new land * [landheight];
    if (la[i]==NULL) { cerr<<"Zuwenig Speicher!\n"; exit(1); }
    for (j=0; j<landheight; j++) (la[i])[j]=NULL;
  }
  
  line[2]=0;
  for (j=0; j<landheight; j++) {
    for (i=0; i<landwidth; i++) {
      eingabe>>line[0];
      eingabe>>line[1];
      sscanf(line,"%d",&typ);
      (la[i])[j]=mw->bs.newLand(typ,i,j);
    }
  }
  
  eingabe>>line;
  if (strcmp("Strassen:",line)!=0) { cerr<<"Falsches Dateiformat von "<<filename<<" !\n"; exit(1); }
  line[2]=0;
  for (j=0; j<landheight; j++) {
    for (i=0; i<landwidth; i++) {
      eingabe>>line[0];
      eingabe>>line[1];
      sscanf(line,"%d",&typ);
      if (typ!=99) (la[i])[j]->setStrasse(typ);
    }
  }
  
  mw->anz_spieler1=0;
  mw->anz_spieler2=0;
  eingabe>>line;
  if (strcmp("Fahrzeuge_Spieler1:",line)!=0) { cerr<<"Falsches Dateiformat von "<<filename<<" !\n"; exit(1); }
  line[2]=0;
  for (j=0; j<landheight; j++) {
    for (i=0; i<landwidth; i++) {
      eingabe>>line[0];
      eingabe>>line[1];
      sscanf(line,"%d",&typ);
      if (typ!=99) {
        (la[i])[j]->einheit_kommt_an(new einheit(this,typ,1));
        (la[i])[j]->zug_beendet(FALSE);
      }
    }
  }
  eingabe>>line;
  if (strcmp("Fahrzeuge_Spieler2:",line)!=0) { cerr<<"Falsches Dateiformat von "<<filename<<" !\n"; exit(1); }
  line[2]=0;
  for (j=0; j<landheight; j++) {
    for (i=0; i<landwidth; i++) {
      eingabe>>line[0];
      eingabe>>line[1];
      sscanf(line,"%d",&typ);
      if (typ!=99) {
        (la[i])[j]->einheit_kommt_an(new einheit(this,typ,2));
        (la[i])[j]->zug_beendet(FALSE);
      }
    }
  }
  
  return TRUE;
  
}

void landschaft::set_offx(int val) {
  offx=-(int)((double)(landwidth*versch_x-width()+versch_x/3)/1000.0*(double)val);
  repaint(FALSE);
}

void landschaft::set_offy(int val) {
  offy=-(int)((double)(landheight*2*versch_y+versch_y-height()+2)/1000.0*(double)val);
  repaint(FALSE);
}

void landschaft::paint_new() {
  repaint(TRUE);
}

void landschaft::mouseReleaseEvent(QMouseEvent *me) {
  
  if (mw->bin_computerspieler()) return;
  
  int		x, y;
  
  x=(me->x()-offx-8)/versch_x;
  if (x<0) x=0;
  if (x>=landwidth) x=landwidth-1;
  
  if (x%2==1) y=(me->y()-offy-versch_y)/(versch_y*2);
  else y=(me->y()-offy)/(versch_y*2);
  if (y<0) y=0;
  if (y>=landheight) y=landheight-1;
  
  if (klick==1) {
    if ((la[x])[y]->oeffnen()==FALSE) {
      if (mw->aktueller_modus()==bewegungsmodus) steuerung_bewegungsmodus(x,y);
      else steuerung_angriffsmodus(x,y);
    }
  } else {
    if (mw->aktueller_modus()==bewegungsmodus) steuerung_bewegungsmodus(x,y);
    else steuerung_angriffsmodus(x,y);
  }
  
  repaint(FALSE);
  
}

void landschaft::bewege_einheit(int sx, int sy, int ex, int ey) {
  
  if ( (sx<0) || (sy<0) || (ex>=landwidth) || (ey>=landheight) ) {
    cerr<<"landschaft::bewege_einheit() Koordinaten liegen ausserhalb des Spielfeldes!\n";
    exit(1);
  }
  
  if ( (sx==ex) && (sy==ey) ) {
    cerr<<"landschaft::bewege_einheit() Keine Bewegung!\n";
    exit(1);
  }
  
  einheit	*e;
  
  e=(la[sx])[sy]->get_einheit();
  if (e==NULL) { cerr<<"landschaft::bewege_einheit() Feld ist unbesetzt!\n"; exit(1); }
  
  if ((la[ex])[ey]->einheit_kommt_an(e)==TRUE) {
    
    (la[sx])[sy]->einheit_geht();
    
    // Meldung an communicator
    // akt_stellplatz bei class depot muss auf den Platz zeigen,
    // wo das zuletzt geparkte Fahrzeug steht !!!
    co->melde_bewegung(sx,sy,ex,ey,(la[sx])[sy]->get_akt_stellplatz(),(la[ex])[ey]->get_akt_stellplatz());
    
  }
  
  erzeugeKarte();
  
}

void landschaft::steuerung_bewegungsmodus(int klickx, int klicky) {
  
  int		i, j;
  einheit	*e;
  land		*l;
  
  if (klick==1) {
    
    e=(la[klickx])[klicky]->get_einheit();
    if (e==NULL) return;
    mw->info(e);
    
    if ((la[klickx])[klicky]->zug_beendet()==TRUE) return;
    if (e->gehoere_zu!=mw->bin_spieler) return;
    
    markiere_erreichbare_felder(klickx,klicky,(double)(e->bewegung),e->flugzeug);
    
    (la[klickx])[klicky]->rahmen(FALSE);
    
    first_klickx=klickx;
    first_klicky=klicky;
    klick=2;
  } else {
    
    l=(la[klickx])[klicky];
    
    e=l->get_einheit();
    if (e!=NULL) mw->info(e);
    
    if (l->rahmen()) {
      e=(la[first_klickx])[first_klicky]->get_einheit();
      if ((l->bin_depot()) && (l->gehoere_zu()!=mw->bin_spieler)) {
        if (e->fusstruppe()) co->melde_uebernahme(klickx,klicky);
        else {
          cerr<<"Uebernahme von Depot nur mit Fusseinheit\n";
          exit(1);
        }
      }
      bewege_einheit(first_klickx,first_klicky,klickx,klicky);
    }
    
    for (i=0; i<landwidth; i++)
      for (j=0; j<landheight; j++)
        if ( (la[i])[j]->rahmen()==TRUE ) (la[i])[j]->rahmen(FALSE);
    
    klick=1;
  }
  
}

void landschaft::steuerung_angriffsmodus(int klickx, int klicky) {
  
  int		i, j, bo;
  einheit	*e1, *e2;
  int		gegenwehr;
  
  if (klick==1) {
    
    e1=(la[klickx])[klicky]->get_einheit();
    if (e1==NULL) return;
    mw->info(e1);
    
    if ((la[klickx])[klicky]->zug_beendet()==TRUE) return;
    if (e1->gehoere_zu!=mw->bin_spieler) return;
    
    bo=FALSE;
    
    for (i=0; i<landwidth; i++) {
      for (j=0; j<landheight; j++) {
        if ((la[i])[j]->bin_depot()==FALSE) {
          e2=(la[i])[j]->get_einheit();
          if (e2!=NULL) {
            if (e2->gehoere_zu!=mw->bin_spieler) {
              if (ist_in_radius(e1->min_schussweite-1,klickx,klicky,i,j)==FALSE) {
                if (ist_in_radius(e1->max_schussweite,klickx,klicky,i,j)==TRUE) {
                  if (e2->flugzeug) {
                    if (e1->feuerkraft_luft>0) {
                      (la[i])[j]->rahmen(TRUE);
                      bo=TRUE;
                    }
                  } else {
                    if (e1->feuerkraft_boden>0) {
                      (la[i])[j]->rahmen(TRUE);
                      bo=TRUE;
                    }
                  }
                }
              }
            }
          }
        }
      }
    }
    first_klickx=klickx;
    first_klicky=klicky;
    if (bo) klick=2;
  } else {
    
    if ( (la[klickx])[klicky]->rahmen()==TRUE ) {
      gegenwehr=ist_in_radius(1,first_klickx,first_klicky,klickx,klicky);
      e1=(la[klickx])[klicky]->get_einheit();
      if (ist_in_radius(e1->min_schussweite-1,first_klickx,first_klicky,klickx,klicky)) gegenwehr=FALSE;
      greife_einheit_an(first_klickx,first_klicky,klickx,klicky,gegenwehr);
    }
    
    for (i=0; i<landwidth; i++)
      for (j=0; j<landheight; j++)
        if ( (la[i])[j]->rahmen()==TRUE ) (la[i])[j]->rahmen(FALSE);
    
    klick=1;
  }
  
}

void landschaft::modus_wechseln() {
  
  int	i, j;
  
  for (i=0; i<landwidth; i++)
    for (j=0; j<landheight; j++)
      (la[i])[j]->modus_wechseln();
  
  klick=1;
  
  erzeugeKarte();
  
  repaint();
  
}

int landschaft::ist_in_radius(int r, int mx, int my, int x, int y) {
  
  int	abs, ly, by;
  
  abs=x-mx;
  if (abs<0) abs=-abs;
  
  if (abs<=r) {
    ly=(r<<1)+1-abs;
    if ( (mx%2)==0 ) {
      // gerade
      by=my-r+(abs>>1);
      if ( (y>=by) && (y<(by+ly)) ) return TRUE;
    } else {
      // ungerade
      by=my+r-(abs>>1);
      if ( (y<=by) && (y>(by-ly)) ) return TRUE;
    }
  }
  
  return FALSE;
  
}

void landschaft::neuer_communicator(communicator *com) {
  co=com;
}

void landschaft::greife_einheit_an(int x, int y, int nx, int ny, int gegenwehr) {
  
  (la[x])[y]->zug_beendet(TRUE);
  
  // Meldung an communicator
  
  co->melde_angriff(x,y,nx,ny,gegenwehr);
  
  erzeugeKarte();
  
}

einheit *landschaft::get_einheit(int x, int y, int platz) {
  return (la[x])[y]->get_einheit(platz);
}

void landschaft::einheit_zerstoert(int x, int y, int platz) {
  (la[x])[y]->einheit_zerstoert(platz);
}

void landschaft::markiere_erreichbare_felder(int sx, int sy, double bew, int flugzeug) {
  
  int		feld_erg[landwidth*landheight],
		x, y;
  double	feld_bew[landwidth*landheight],
		schritte[landwidth*landheight];
  einheit	*e, *ee;
  
  ee=(la[sx])[sy]->get_einheit();
  if (ee==NULL) { cerr<<"interner Fehler\n"; exit(1); }
  
  // Felder fuellen
  for (x=0; x<landwidth; x++) {
    for (y=0; y<landheight; y++) {
      schritte[y*landwidth+x]=0.0;
      e=(la[x])[y]->get_einheit();
      if ((la[x])[y]->platz_frei()) {
        if ((la[x])[y]->bin_depot()) {
          if ((la[x])[y]->gehoere_zu()!=mw->bin_spieler)
            if (ee->fusstruppe()) feld_bew[y*landwidth+x]=(la[x])[y]->befahrbarkeit;
            else feld_bew[y*landwidth+x]=5000;
          else feld_bew[y*landwidth+x]=(la[x])[y]->befahrbarkeit;
        } else feld_bew[y*landwidth+x]=(la[x])[y]->befahrbarkeit;
      } else feld_bew[y*landwidth+x]=e->gehoere_zu*1000;	// Achtung: e kann NULL sein
      feld_erg[y*landwidth+x]=FALSE;
    }
  }
  
  markiere_erreichbare_felder_rek(feld_bew,feld_erg,schritte,sx,sy,bew+1.0,flugzeug);
  
  // Informationen zurueckuebertragen
  for (x=0; x<landwidth; x++) {
    for (y=0; y<landheight; y++) {
      if (feld_erg[y*landwidth+x])
        (la[x])[y]->rahmen(TRUE);
    }
  }
  
}

void landschaft::markiere_erreichbare_felder_rek(double *feld_bew, int *feld_erg, double *schritte, int sx, int sy, double bew, int flugzeug) {
  
  if (bew<=schritte[sy*landwidth+sx]) return;
  schritte[sy*landwidth+sx]=bew;
  
  // markiert die fuer eine Einheit erreichbaren Felder
  
  int		x, y, nx, ny, a;
  double	v;
  einheit	*e;
  
  a=sy*landwidth+sx;
  
  if ((int)(feld_bew[a]+0.5)==mw->bin_spieler*1000) {
    if (flugzeug) bew-=1.0;
    else bew-=(la[sx])[sy]->befahrbarkeit;
  } else {
    if (feld_bew[a]>999) {
      return;
    } else {
      if (flugzeug) bew-=1.0;
      else bew-=feld_bew[a];
      feld_erg[a]=TRUE;
    }
    // wenn eingeschlossen, dann return
    // vertikal
    e=nachbar(sx,sy,nachbar_o);
    if (e!=NULL) {
      if (e->gehoere_zu!=mw->bin_spieler) {
        e=nachbar(sx,sy,nachbar_u);
        if (e!=NULL) {
          if (e->gehoere_zu!=mw->bin_spieler) return;
        }
      }
    }
    // horizontal 1
    e=nachbar(sx,sy,nachbar_ol);
    if (e!=NULL) {
      if (e->gehoere_zu!=mw->bin_spieler) {
        e=nachbar(sx,sy,nachbar_ur);
        if (e!=NULL) {
          if (e->gehoere_zu!=mw->bin_spieler) return;
        }
      }
    }
    // horizontal 2
    e=nachbar(sx,sy,nachbar_or);
    if (e!=NULL) {
      if (e->gehoere_zu!=mw->bin_spieler) {
        e=nachbar(sx,sy,nachbar_ul);
        if (e!=NULL) {
          if (e->gehoere_zu!=mw->bin_spieler) return;
        }
      }
    }
  }
  
  if (bew<=0) return;
  
  // oben
  if (nachbar_koords(sx,sy,nachbar_o,&x,&y)==TRUE) {
    v=feld_bew[y*landwidth+x];
    if ((v<900) || ((int)(v+0.5)==mw->bin_spieler*1000)) {
      if (nachbar_koords(sx,sy,nachbar_ol,&nx,&ny)==TRUE) {
        v=feld_bew[ny*landwidth+nx];
        if (((int)(v+0.5)==anderer_spieler*1000) || ((int)(v+0.5)==1111)) {
          if (nachbar_koords(sx,sy,nachbar_or,&nx,&ny)==TRUE) {
            v=feld_bew[ny*landwidth+nx];
            if (((int)(v+0.5)==anderer_spieler*1000) || ((int)(v+0.5)==1111)) {
              if (feld_bew[y*landwidth+x]<900) feld_erg[y*landwidth+x]=TRUE;
            } else {
              markiere_erreichbare_felder_rek(feld_bew,feld_erg,schritte,x,y,bew,flugzeug);
            }
          } else {
            markiere_erreichbare_felder_rek(feld_bew,feld_erg,schritte,x,y,bew,flugzeug);
          }
        } else {
          markiere_erreichbare_felder_rek(feld_bew,feld_erg,schritte,x,y,bew,flugzeug);
        }
      } else {
        markiere_erreichbare_felder_rek(feld_bew,feld_erg,schritte,x,y,bew,flugzeug);
      }
    }
  }
  
  // unten
  if (nachbar_koords(sx,sy,nachbar_u,&x,&y)==TRUE) {
    v=feld_bew[y*landwidth+x];
    if ((v<900) || ((int)(v+0.5)==mw->bin_spieler*1000)) {
      if (nachbar_koords(sx,sy,nachbar_ul,&nx,&ny)==TRUE) {
        v=feld_bew[ny*landwidth+nx];
        if (((int)(v+0.5)==anderer_spieler*1000) || ((int)(v+0.5)==1111)) {
          if (nachbar_koords(sx,sy,nachbar_ur,&nx,&ny)==TRUE) {
            v=feld_bew[ny*landwidth+nx];
            if (((int)(v+0.5)==anderer_spieler*1000) || ((int)(v+0.5)==1111)) {
              if (feld_bew[y*landwidth+x]<900) feld_erg[y*landwidth+x]=TRUE;
            } else {
              markiere_erreichbare_felder_rek(feld_bew,feld_erg,schritte,x,y,bew,flugzeug);
            }
          } else {
            markiere_erreichbare_felder_rek(feld_bew,feld_erg,schritte,x,y,bew,flugzeug);
          }
        } else {
          markiere_erreichbare_felder_rek(feld_bew,feld_erg,schritte,x,y,bew,flugzeug);
        }
      } else {
        markiere_erreichbare_felder_rek(feld_bew,feld_erg,schritte,x,y,bew,flugzeug);
      }
    }
  }
  
  // oben links
  if (nachbar_koords(sx,sy,nachbar_ol,&x,&y)==TRUE) {
    v=feld_bew[y*landwidth+x];
    if ((v<900) || ((int)(v+0.5)==mw->bin_spieler*1000)) {
      if (nachbar_koords(sx,sy,nachbar_o,&nx,&ny)==TRUE) {
        v=feld_bew[ny*landwidth+nx];
        if (((int)(v+0.5)==anderer_spieler*1000) || ((int)(v+0.5)==1111)) {
          if (nachbar_koords(sx,sy,nachbar_ul,&nx,&ny)==TRUE) {
            v=feld_bew[ny*landwidth+nx];
            if (((int)(v+0.5)==anderer_spieler*1000) || ((int)(v+0.5)==1111)) {
              if (feld_bew[y*landwidth+x]<900) feld_erg[y*landwidth+x]=TRUE;
            } else {
              markiere_erreichbare_felder_rek(feld_bew,feld_erg,schritte,x,y,bew,flugzeug);
            }
          } else {
            markiere_erreichbare_felder_rek(feld_bew,feld_erg,schritte,x,y,bew,flugzeug);
          }
        } else {
          markiere_erreichbare_felder_rek(feld_bew,feld_erg,schritte,x,y,bew,flugzeug);
        }
      } else {
        markiere_erreichbare_felder_rek(feld_bew,feld_erg,schritte,x,y,bew,flugzeug);
      }
    }
  }
  
  // oben rechts
  if (nachbar_koords(sx,sy,nachbar_or,&x,&y)==TRUE) {
    v=feld_bew[y*landwidth+x];
    if ((v<900) || ((int)(v+0.5)==mw->bin_spieler*1000)) {
      if (nachbar_koords(sx,sy,nachbar_o,&nx,&ny)==TRUE) {
        v=feld_bew[ny*landwidth+nx];
        if (((int)(v+0.5)==anderer_spieler*1000) || ((int)(v+0.5)==1111)) {
          if (nachbar_koords(sx,sy,nachbar_ur,&nx,&ny)==TRUE) {
            v=feld_bew[ny*landwidth+nx];
            if (((int)(v+0.5)==anderer_spieler*1000) || ((int)(v+0.5)==1111)) {
              if (feld_bew[y*landwidth+x]<900) feld_erg[y*landwidth+x]=TRUE;
            } else {
              markiere_erreichbare_felder_rek(feld_bew,feld_erg,schritte,x,y,bew,flugzeug);
            }
          } else {
            markiere_erreichbare_felder_rek(feld_bew,feld_erg,schritte,x,y,bew,flugzeug);
          }
        } else {
          markiere_erreichbare_felder_rek(feld_bew,feld_erg,schritte,x,y,bew,flugzeug);
        }
      } else {
        markiere_erreichbare_felder_rek(feld_bew,feld_erg,schritte,x,y,bew,flugzeug);
      }
    }
  }
  
  // unten links
  if (nachbar_koords(sx,sy,nachbar_ul,&x,&y)==TRUE) {
    v=feld_bew[y*landwidth+x];
    if ((v<900) || ((int)(v+0.5)==mw->bin_spieler*1000)) {
      if (nachbar_koords(sx,sy,nachbar_ol,&nx,&ny)==TRUE) {
        v=feld_bew[ny*landwidth+nx];
        if (((int)(v+0.5)==anderer_spieler*1000) || ((int)(v+0.5)==1111)) {
          if (nachbar_koords(sx,sy,nachbar_u,&nx,&ny)==TRUE) {
            v=feld_bew[ny*landwidth+nx];
            if (((int)(v+0.5)==anderer_spieler*1000) || ((int)(v+0.5)==1111)) {
              if (feld_bew[y*landwidth+x]<900) feld_erg[y*landwidth+x]=TRUE;
            } else {
              markiere_erreichbare_felder_rek(feld_bew,feld_erg,schritte,x,y,bew,flugzeug);
            }
          } else {
            markiere_erreichbare_felder_rek(feld_bew,feld_erg,schritte,x,y,bew,flugzeug);
          }
        } else {
          markiere_erreichbare_felder_rek(feld_bew,feld_erg,schritte,x,y,bew,flugzeug);
        }
      } else {
        markiere_erreichbare_felder_rek(feld_bew,feld_erg,schritte,x,y,bew,flugzeug);
      }
    }
  }
  
  // unten rechts
  if (nachbar_koords(sx,sy,nachbar_ur,&x,&y)==TRUE) {
    v=feld_bew[y*landwidth+x];
    if ((v<900) || ((int)(v+0.5)==mw->bin_spieler*1000)) {
      if (nachbar_koords(sx,sy,nachbar_or,&nx,&ny)==TRUE) {
        v=feld_bew[ny*landwidth+nx];
        if (((int)(v+0.5)==anderer_spieler*1000) || ((int)(v+0.5)==1111)) {
          if (nachbar_koords(sx,sy,nachbar_u,&nx,&ny)==TRUE) {
            v=feld_bew[ny*landwidth+nx];
            if (((int)(v+0.5)==anderer_spieler*1000) || ((int)(v+0.5)==1111)) {
              if (feld_bew[y*landwidth+x]<900) feld_erg[y*landwidth+x]=TRUE;
            } else {
              markiere_erreichbare_felder_rek(feld_bew,feld_erg,schritte,x,y,bew,flugzeug);
            }
          } else {
            markiere_erreichbare_felder_rek(feld_bew,feld_erg,schritte,x,y,bew,flugzeug);
          }
        } else {
          markiere_erreichbare_felder_rek(feld_bew,feld_erg,schritte,x,y,bew,flugzeug);
        }
      } else {
        markiere_erreichbare_felder_rek(feld_bew,feld_erg,schritte,x,y,bew,flugzeug);
      }
    }
  }
  
}

einheit *landschaft::nachbar(int x, int y, int wo) {
  
  switch(wo) {
    case nachbar_o:
      if (y>0) return (la[x])[y-1]->get_einheit();
      return NULL;
    case nachbar_u:
      if (y<landheight-1) return (la[x])[y+1]->get_einheit();
      return NULL;
    case nachbar_ol:
      if ((x%2)==0) {
        if ( (x>0) && (y>0) ) return (la[x-1])[y-1]->get_einheit();
      } else {
        if (x>0) return (la[x-1])[y]->get_einheit();
      }
      return NULL;
    case nachbar_ul:
      if (x%2==0) {
        if (x>0) return (la[x-1])[y]->get_einheit();
      } else {
        if ( (x>0) && (y<landheight-1) ) return (la[x-1])[y+1]->get_einheit();
      }
      return NULL;
    case nachbar_or:
      if (x%2==0) {
        if ( (x<landwidth-1) && (y>0) ) return (la[x+1])[y-1]->get_einheit();
      } else {
        if (x<landwidth-1) return (la[x+1])[y]->get_einheit();
      }
      return NULL;
    case nachbar_ur:
      if (x%2==0) {
        if (x<landwidth-1) return (la[x+1])[y]->get_einheit();
      } else {
        if ( (x<landwidth-1) && (y<landheight-1) ) return (la[x+1])[y+1]->get_einheit();
      }
      return NULL;
    default:
      cerr<<"landschaft::nachbar() unbekannte Richtung!\n";
      exit(1);
  }
  
}

int landschaft::nachbar_koords(int x, int y, int wo, int *kx, int *ky) {
  
  switch(wo) {
    case nachbar_o:
      if (y>0) { *kx=x; *ky=y-1; return TRUE; }
      return FALSE;
    case nachbar_u:
      if (y<landheight-1) { *kx=x; *ky=y+1; return TRUE; }
      return FALSE;
    case nachbar_ol:
      if (x%2==0) {
        if ( (x>0) && (y>0) ) { *kx=x-1; *ky=y-1; return TRUE; }
      } else {
        if (x>0) { *kx=x-1; *ky=y; return TRUE; }
      }
      return FALSE;
    case nachbar_ul:
      if (x%2==0) {
        if (x>0) { *kx=x-1; *ky=y; return TRUE; }
      } else {
        if ( (x>0) && (y<landheight-1) ) { *kx=x-1; *ky=y+1; return TRUE; }
      }
      return FALSE;
    case nachbar_or:
      if (x%2==0) {
        if ( (x<landwidth-1) && (y>0) ) { *kx=x+1; *ky=y-1; return TRUE; }
      } else {
        if (x<landwidth-1) { *kx=x+1; *ky=y; return TRUE; }
      }
      return FALSE;
    case nachbar_ur:
      if (x%2==0) {
        if (x<landwidth-1) { *kx=x+1; *ky=y; return TRUE; }
      } else {
        if ( (x<landwidth-1) && (y<landheight-1) ) { *kx=x+1; *ky=y+1; return TRUE; }
      }
      return FALSE;
    default:
      cerr<<"landschaft::nachbar() unbekannte Richtung!\n";
      exit(1);
  }
  
}

void landschaft::landschaft_groesse(int *w, int *h) {
  *w=landwidth;
  *h=landheight;
}

void landschaft::bewegung_ausfuehren(int sx, int sy, int ex, int ey, int platz_start, int platz_ziel) {
  
  if ( (sx<0) || (sy<0) || (ex>=landwidth) || (ey>=landheight) ) {
    cerr<<"landschaft::bewege_einheit() Koordinaten liegen ausserhalb des Spielfeldes!\n";
    exit(1);
  }
  
  if ( (sx==ex) && (sy==ey) ) {
    cerr<<"landschaft::bewege_einheit() Keine Bewegung!\n";
    exit(1);
  }
  
  einheit	*e;
  
  e=(la[sx])[sy]->get_einheit(platz_start);
  if (e==NULL) {
    // Einheit wurde zerstoert
    return;
  }
  
  if ((la[ex])[ey]->einheit_kommt_an(e,platz_ziel)==TRUE) (la[sx])[sy]->einheit_geht(platz_start);
  else { cerr<<"interner Fehler! bewegung_ausfuehren()\n"; exit(1); }
  
}

land *landschaft::get_land(int x, int y) {
  
  if ((x>=0) && (x<landwidth) && (y>=0) && (y<landheight))  return (la[x])[y];
  else return NULL;
  
}

void landschaft::steuerung_reset() {
  klick=1;
}

void landschaft::zaehleFahrzeuge() {
  
  int		a1=0, a2=0, x, y, i;
  einheit	*e;
  land		*l;
  
  for (x=0; x<landwidth; x++) {
    for (y=0; y<landheight; y++) {
      l=get_land(x,y);
      if (l->bin_depot()) {
        for (i=0; i<anz_inhalt; i++) {
          e=l->get_einheit(i);
          if (e!=NULL) {
            if (e->gehoere_zu==1) a1++;
            else a2++;
          }
        }
      } else {
        e=l->get_einheit();
        if (e!=NULL) {
          if (e->gehoere_zu==1) a1++;
          else a2++;
        }
      }
    }
  }
  
  mw->anz_spieler1=a1;
  mw->anz_spieler2=a2;
  
}

void landschaft::erzeugeKarte() {
  
  int		x, y, px, py;
  land		*l;
  einheit	*e;
  QPainter	pa;
  QBrush	br(SolidPattern);
  QColor	lightBlue(120,200,255),
		lightRed(255,120,150),
		darkGray(100,100,100);
  
  pa.begin(karte);
  pa.eraseRect(0,0,karte->width(),karte->height());
  for (x=0; x<landwidth; x++) {
    for (y=0; y<landheight; y++) {
      l=get_land(x,y);
      if ((l->bin_depot()) || (l->bin_factory()) || (l->bin_headquarter())) {
        if (l->gehoere_zu()==1) {
          pa.setPen(lightRed);
          br.setColor(lightRed);
        } else {
          pa.setPen(lightBlue);
          br.setColor(lightBlue);
        }
      } else if (l->bin_depotumr()) {
        pa.setPen(gray);
        br.setColor(gray);
      } else {
        e=l->get_einheit();
        if (e==NULL) {
          if (l->bin_strasse()) {
            pa.setPen(darkGray);
            br.setColor(darkGray);
          } else {
            pa.setPen(darkGreen);
            br.setColor(darkGreen);
          }
        } else {
          if (l->zug_beendet()) {
            pa.setPen(black);
            br.setColor(black);
          } else if (e->gehoere_zu==1) {
            pa.setPen(red);
            br.setColor(red);
          } else {
            pa.setPen(blue);
            br.setColor(blue);
          }
        }
      }
      px=x*karte_rect_size;
      py=y*karte_rect_size+(x%2)*karte_rect_size/2;
      pa.fillRect(px,py,karte_rect_size,karte_rect_size,br);
    }
  }
  pa.end();
  
  mw->redraw();
  
}


