
#include <qapplication.h>
#include <stdio.h>
#include <string.h>
#include <fstream.h>

#include "main_win.hh"
#include "communicator.hh"
#include "computerspieler.hh"

char levelName[256];

main_win::main_win(int spieler, computerspieler *csp, QWidget *parent, const char *name)
  : QWidget(parent,name),
    kampf(this,"battle screen on","battle screen off"),
    bs(this),
    karte(this)
  {
  
  char		fname[256];
  
  if (spieler<1) { cerr<<"Spieler muss >=1 sein!\n"; exit(1); }
  
  can_change_mod=FALSE;
  co=NULL;
  
  bin_spieler=spieler;
  
  ende=NULL;
  ls=NULL;
  modus_label=NULL;
  mod_wechsel=NULL;
  status_label=NULL;
  info_label=NULL;
  
  if (spieler==1) {
    if (csp==NULL) setCaption("XB.I.C. player 1 red");
    else setCaption("XB.I.C. computer 1 red");
  } else {
    if (csp==NULL) setCaption("XB.I.C. player 2 blue");
    else setCaption("XB.I.C. computer 2 blue");
  }
  
  modus_label=new QLabel(this,"modus_label");
  modus_label->move(10,40);
  modus_label->resize(390,20);
  modus_label->setFrameStyle(QFrame::Panel | QFrame::Raised);
  modus_label->setAlignment(AlignHCenter | AlignVCenter);
  
  if (csp==NULL) {
    mod_wechsel=new QPushButton("change mode",this);
    mod_wechsel->move(410,40);
    mod_wechsel->resize(100,20);
    connect(mod_wechsel,SIGNAL(clicked()),this,SLOT(modus_change_pressed()));
  }
  
  strcpy(fname,levelName);
  strcat(fname,"/land.lev");
  ls=new landschaft(fname,this,"ls");
  ls->move(10,70);
  
  status_label=new QLabel(this,"status_label");
  status_label->move(410,70);
  status_label->resize(100,50);
  status_label->setFrameStyle(QFrame::Panel | QFrame::Raised);
  status_label->setAlignment(AlignHCenter | AlignVCenter);
  
  info_label=new QLabel(this,"info_label");
  info_label->move(410,130);
  info_label->resize(100,190);
  info_label->setFrameStyle(QFrame::Panel | QFrame::Raised);
  info_label->setAlignment(AlignVCenter | AlignLeft);
  
  ende=new QPushButton("quit",this);
  ende->move(10,360);
  ende->resize(100,20);
  
  connect(ende,SIGNAL(clicked()),this,SLOT(exitProg()));
  
  if (spieler==1) set_modus(bewegungsmodus);
  else set_modus(angriffsmodus);
  
  cs=csp;
  if (cs!=NULL) {
    cs->init(this);
    startTimer(1000);
  }
  
  setMinimumSize(600,480);
  
  ls->erzeugeKarte();
  
  paintEvent(NULL);
  
}

main_win::~main_win() {
  
  if (ende!=NULL) delete ende;
  if (ls!=NULL) delete ls;
  if (modus_label!=NULL) delete modus_label;
  if (mod_wechsel!=NULL) delete mod_wechsel;
  if (status_label!=NULL) delete status_label;
  if (info_label!=NULL) delete info_label;
  if (co!=NULL) delete co;
  
}

int main_win::aktueller_modus() { return modus; }

void main_win::set_modus(int new_modus) {
  
  modus=new_modus;
  if (modus==bewegungsmodus) modus_label->setText("move");
  else modus_label->setText("attack");
  
  show_status();
  
}

void main_win::modus_wechseln() {
  
  ls->zaehleFahrzeuge();
  
  if (modus==bewegungsmodus) set_modus(angriffsmodus);
  else set_modus(bewegungsmodus);
  
  ls->modus_wechseln();
  show_status();
  
  if (anz_spieler2<=0) victory("player 2 has lost all units, victory player 1!\n");
  if (anz_spieler1<=0) victory("player 1 has lost all units, victory player 2!\n");
  
  can_change_mod=FALSE;
  
}

void main_win::show_status() {
  
  char	str[256];
  
  sprintf(str,"player 1: %d\nplayer 2: %d",anz_spieler1,anz_spieler2);
  status_label->setText(str);
  
}

void main_win::info(einheit *e) {
  
  if (e==NULL) return;
  
  char	str[512];
  
  if (e->gehoere_zu==1) strcpy(str,"player 1\n");
  else strcpy(str,"player 2\n");
  sprintf(str+strlen(str),"unit\n%s\n",e->gruppe);
  sprintf(str+strlen(str),"number         %d\n",e->anzahl);
  sprintf(str+strlen(str),"training       %d\n",e->ausbildung);
  sprintf(str+strlen(str),"min shot range %d\n",e->min_schussweite);
  sprintf(str+strlen(str),"max shot range %d\n",e->max_schussweite);
  sprintf(str+strlen(str),"move range     %d\n",e->bewegung);
  sprintf(str+strlen(str),"armour         %d\n",e->panzerung);
  sprintf(str+strlen(str),"air            %d\n",e->feuerkraft_luft);
  sprintf(str+strlen(str),"ground         %d",e->feuerkraft_boden);
  
  info_label->setText(str);
  
}

landschaft *main_win::get_landschaft() {
  if (ls==NULL) { cerr<<"class landschaft ist noch nicht initialisiert!\n"; exit(1); }
  return ls;
}

void main_win::modus_change_pressed() {
  
  // ###
  // nicht die beste Loesung
  // solange kein Kampf stattfindet haeufen sich diese Meldungen
  // an, eventuell bis die queue voll ist
  co->sendKampfbildschirmMsg(kampf.status());
  // ###
  
  if (can_change_mod==FALSE) {
    can_change_mod=TRUE;
    if (modus==bewegungsmodus) modus_label->setText("move, wait for other player");
    else modus_label->setText("attack, wait for other player");
  } else {
    can_change_mod=FALSE;
    if (modus==bewegungsmodus) modus_label->setText("move");
    else modus_label->setText("attack");
  }
  
  co->bereit_fuer_moduswechsel(can_change_mod);
  
}

void main_win::neuer_communicator(communicator *com) {
  co=com;
}

void main_win::paintEvent(QPaintEvent *) {
  
  show_status();
  karte.setPixmap(*(ls->karte));
  
  modus_label->setGeometry(10,10,width()-180,20);
  ls->setGeometry(10,40,width()-210,height()-80);
  
  if (cs==NULL) mod_wechsel->setGeometry(width()-160,10,150,20);
  kampf.setGeometry(width()-160,40,150,20);
  status_label->setGeometry(width()-160,70,150,50);
  info_label->setGeometry(width()-160,130,150,160);
  ende->setGeometry(width()-160,height()-30,150,20);
  karte.setGeometry(width()-160,300,150,height()-340);
  
}

void main_win::closeEvent(QCloseEvent *) {
  
  exitProg();
  
}

void main_win::timerEvent(QTimerEvent *) {
  
  if (co->warten_auf_wechsel()==FALSE) {
    if (aktueller_modus()==angriffsmodus) cs->angreifen();
    else cs->bewegen();
  }
  
}

int main_win::bin_computerspieler() {
  if (cs==NULL) return FALSE;
  else return TRUE;
}

void main_win::zaehleFahrzeuge() {
  ls->zaehleFahrzeuge();
  paintEvent(NULL);
}

void main_win::redraw() {
  paintEvent(NULL);
}

void main_win::exitProg() {
  
  exit(0);
  
}

void main_win::victory(char *txt) {
  co->victory(txt);
}
